@@ -251,7 +251,6 @@ class Regatron(Display):
 
         self.flashHistoryTable.clearContents()
         values = kwargs["value"][:-1]
-        data = []
 
         self.flashHistoryTable.setRowCount(len(values))
         i = 0
@@ -331,10 +330,12 @@ class Regatron(Display):
         self.sysStatus.setVisible(self.isMaster)
 
         self.warnByte.channel = "ca://{}{}".format(
-            self.macros_dict["P"], ":GenWarn-Mon" if self.isMaster else ":ModWarnGroup-Mon",
+            self.macros_dict["P"],
+            ":GenWarn-Mon" if self.isMaster else ":ModWarnGroup-Mon",
         )
         self.errorByte.channel = "ca://{}{}".format(
-            self.macros_dict["P"], ":Intlk-Mon" if self.isMaster else ":ModErrGroup-Mon",
+            self.macros_dict["P"],
+            ":GenIntlk-Mon" if self.isMaster else ":ModErrGroup-Mon",
         )
 
     def changeProtectionLevel(self):
